<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2008 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
die("<br><strong>This script is only meant to run at the command line.</strong>");
}

/* We are not talking to the browser */
$no_http_headers = true;

$dir = dirname(__FILE__);
chdir($dir);

if (substr_count(strtolower($dir), 'configstore')) {
        chdir('../../');
}

if (file_exists("./include/global.php")) {
	include_once("./include/global.php");
} else {
	include_once("./include/config.php");
}

include_once($config['base_path'] . '/plugins/configstore/functions.php');
include_once($config['base_path'] . '/plugins/configstore/include/logger.class.php');

global $log;
$log = new Logging();

plugin_configstore_retention ();

/* Time we allow the poller to start past the designated poller start time */
$grace_period = '3600';

$fail_report = array();
$t = time();

/* Time we wish to start the poller each day */
$poller_start = read_config_option('plugin_configstore_backup_time');
$stime = strtotime($poller_start);
$h = date('H', $stime);
$m = date('i', $stime);
$s = date('s', $stime);

/* Derive the correct time in seconds from our designated start time, based on todays date.
   We also need to subtract the current time in seconds from the desired poller start time
   we derive, to know whether or not it is time to start the polling cycle. */
$now = getDate();
$current_time=mktime($now["hours"],$now["minutes"],$now["seconds"],$now["mon"],$now["mday"],$now["year"]);
$poll_time=mktime($h,$m,$s,$now["mon"],$now["mday"],$now["year"]);
$time_counter = $poll_time - $current_time;

$devices_polled = 0;
/* Do our normal polling cycle */
if ( ($time_counter < 0) && ($time_counter > -$grace_period) ) {
	$devices = db_fetch_assoc("SELECT id FROM plugin_configstore_devices WHERE enabled = 'on' AND ($t - (schedule * 86400) - $grace_period > lastattempt OR lastattempt = '0')");
	$disabled_devices = db_fetch_assoc("SELECT * FROM plugin_configstore_devices WHERE enabled = ''");

	if (!empty($devices)) {
		/* Indicate start of poller */
		db_execute("REPLACE INTO settings (name, value) VALUES ('plugin_configstore_running', 1)");

		foreach ($devices as $id) {
			$id = $id['id'];
			$device = db_fetch_row("SELECT * FROM plugin_configstore_devices WHERE id = $id");
			$result = plugin_configstore_download_config($device);
			$devices_polled++;
			if ($result) {
				db_execute("UPDATE plugin_configstore_devices SET backupresult='0', failedcount='0' WHERE id = " . $device['id']);
			} else {
                        	$failedcount = db_fetch_cell("SELECT failedcount FROM plugin_configstore_devices WHERE id = " . $device['id']);
                        	$failedcount++;
				$fail_report[] = array ('hostname'	=> $device['hostname'],
							'ipaddress'	=> $device['ipaddress'],
							'failcount'	=> $failedcount);

				if ($failedcount > 2) {
                                	db_execute("UPDATE plugin_configstore_devices SET backupresult='1', failedcount='0', enabled='' WHERE id = " . $device['id']);
                        	} else {
                                	db_execute("UPDATE plugin_configstore_devices SET backupresult='1', failedcount='$failedcount' WHERE id = " . $device['id']);
                        	}
			}
		}

		if ($devices_polled > 0) {
			$success_count = $devices_polled - count($fail_report);
			$failed_count = count($fail_report);
			$disabled_count = count($disabled_devices);
			$newLine = "\r\n";

			$run_time = time() - $t;
			$run_times = Sec2Time($run_time);	

			$hms = "";

			if ( (isset($run_times['hours'])) && ($run_times['hours'] !== 0) ) {
        			$hms .= $run_times['hours'] . " hours ";
			}
			if ( (isset($run_times['days'])) && ($run_times['minutes'] !== 0) ) {
        			$hms .= $run_times['minutes'] . " minutes ";
			}
			if ( (isset($run_times['seconds'])) && ($run_times['seconds'] !== 0) ) {
        			$hms .= $run_times['seconds'] . " seconds ";
			}
			trim($hms);

			$message = "<html><body style='font-family: helvetica'>";
			$message .= "<div width=75% style='margin-top: 10px; margin-bottom: 25px'>";
			$message .= "<div style='margin-top: 20px; margin-bottom: 10px'>";
			$message .= "<table cellpadding=0 cellspacing=0 border=0><tr width=100% col=3>";
			$message .= "<td colspan=3><center><p style='font-size: 12pt; color: #333366; margin-bottom: 20px'><strong>ConfigStore Network Backup Report</strong></p></center></td>";
			$message .= "</tr><tr width=100% col=3><td colspan=1 width=25%>&nbsp;</td>";
			$message .= "<td colspan=1 width=50%><left><p style='font-size: 10pt; color: #333333; margin-bottom: -10px'><strong>Statistics:</strong></p></left></td>";
			$message .= "<td colspan=1 width=25%>&nbsp;</td></tr><tr width=100% col=3><td colspan=1 width=25%>&nbsp;</td>";
			$message .= "<td colspan=1 width=50%><left><p style='font-size: 8pt; color: #333333; margin-bottom: -10px'><strong>$devices_polled</strong> devices polled in $hms.</p></left></td>";
			$message .= "<td colspan=1 width=25%>&nbsp;</td></tr><tr width=100% col=3><td colspan=1 width=25%>&nbsp;</td>";
			$message .= "<td colspan=1 width=50%><left><p style='font-size: 8pt; color: #333333; margin-bottom: -10px'><strong>$success_count</strong> devices backed up.</p></left></td>";
			$message .= "<td colspan=1 width=25%>&nbsp;</td></tr><tr width=100% col=3><td colspan=1 width=25%>&nbsp;</td>";
			$message .= "<td colspan=1 width=50%><left><p style='font-size: 8pt; color: #333333; margin-bottom: -10px'><strong>$failed_count</strong> devices failed to back up.</p></left></td>";
			$message .= "<td colspan=1 width=25%>&nbsp;</td></tr><tr width=100% col=3><td colspan=1 width=25%>&nbsp;</td>";
			$message .= "<td colspan=1 width=50%><left><p style='font-size: 8pt; color: #333333; margin-bottom: -10px'><strong>$disabled_count</strong> devices are disabled.</p></left></td></tr></table></div>";

			if (count($fail_report) > 0) {
				$message .= "<div style='margin-top: 20px; margin-bottom: 10px'>";
				$message .= "<table border=0 cellpadding=0 cellspacing=10>";
				$message .= "<tr width=100% col=5>";
				$message .= "<td colspan=5><center><p style='font-size: 10pt; color: #333366; margin-bottom: 0px'><strong>Failed Backups Report:</strong></p></center></td>";
				$message .= "</tr><tr width=100% col=5><td colspan=1 width=5%>&nbsp;</td>";
				$message .= "<td colspan=1><p style='font-size: 8pt; color: #333333; border-bottom: 1px dotted #333333'><strong>Hostname</strong></p></td>";
				$message .= "<td colspan=1><p style='font-size: 8pt; color: #333333; border-bottom: 1px dotted #333333'><strong>IP Address</strong></p></td>";
				$message .= "<td colspan=1><p style='font-size: 8pt; color: #333333; border-bottom: 1px dotted #333333'><strong>Status</strong></p></td>";
				$message .= "<td colspan=1><p style='font-size: 8pt; color: #333333; border-bottom: 1px dotted #333333'><strong>Inspect Failure</strong></p></td></tr>";
				foreach ($fail_report as $f) {
					$status = '';
					if ($failedcount > 2) { 
						$status = "disabled"; 
					} else { 
						$status = "failed"; 
					}
						$message .= "<tr width=100% col=5><td colspan=1 width=5%>&nbsp;</td>";
						$message .= "<td colspan=1><p style='font-size: 8pt; color: #333333;'>" . $f['hostname'] . "</p></td>";
						$message .= "<td colspan=1><p style='font-size: 8pt; color: #333333;'>" . $f['ipaddress'] . "</p></td>";
					if ($status == 'disabled') {
						$message .= "<td colspan=1><p style='font-size: 8pt; color: #cc0000;'>" . $status . "</p></td>";
					} else {
						$message .= "<td colspan=1><p style='font-size: 8pt; color: #333333;'>" . $status . "[" . $failedcount . "]" . "</p></td>";
					}
					$message .= "<td colspan=1><p style='font-size: 8pt; color: #333333;'>" . '<a href="http://net10/cacti/plugins/configstore/configstore_devices.php?device_status=-1&device_type=-1&rows=-1&filter=' . $f['hostname'] . '&go_x=Go&page=1">' . "Inspect" . "</a>" . "</p></td>";
					$message .= "</tr>";
				}
			} else {
				$message .= "<div style='margin-top: 20px; margin-bottom: 10px'>";
				$message .= "<table border=0 cellpadding=0 cellspacing=10>";
			}

                        foreach ($disabled_devices as $d) {
                        	$message .= "<tr width=100% col=5><td colspan=1 width=5%>&nbsp;</td>";
                                $message .= "<td colspan=1><p style='font-size: 8pt; color: #333333;'>" . $d['hostname'] . "</p></td>";
                                $message .= "<td colspan=1><p style='font-size: 8pt; color: #333333;'>" . $d['ipaddress'] . "</p></td>";
				$message .= "<td colspan=1><p style='font-size: 8pt; color: #cc0000;'>disabled</p></td>";
                                $message .= "<td colspan=1><p style='font-size: 8pt; color: #333333;'>" . '<a href="http://net10/cacti/plugins/configstore/configstore_devices.php?device_status=-1&device_type=-1&rows=-1&filter=' . $d['hostname'] . '&go_x=Go&page=1">' . "Inspect" . "</a>" . "</p></td>";
                                $message .= "</tr>";
                        }
			$message .= "</table></div>";
			$message .= "</div></body></html>";
	
			$to = read_config_option('plugin_configstore_email_to');
			$from = read_config_option('plugin_configstore_email_from');
			$name_from = read_config_option('plugin_configstore_email_name_from');
			$name_to = read_config_option('plugin_configstore_email_name_to');
			$subject = read_config_option('plugin_configstore_email_subject');
	
			configstore_sendmail($from, $name_from, $to, $name_to, $subject, $message);
	
			cacti_log("$success_count devices backed up and $failed_count devices failed in " . $run_time . " seconds.", true, "Configstore");

		}
		/* Indicate poller stop */
                db_execute("REPLACE INTO settings (name, value) VALUES ('plugin_configstore_running', 0)");
	}	
	/* Check to see if any configs aged out */
	plugin_configstore_retention ();
} else {
	/* Scrape up the devices that failed backups last run, and that have not been disabled,
   	and that have not been polled within the last "grace period". */
	$failed = db_fetch_assoc("SELECT id FROM plugin_configstore_devices WHERE enabled = 'on' AND backupresult = '1' AND $t - $grace_period > lastattempt");
	if ( (!empty($failed)) && (($time_counter > $grace_period) || ($time_counter < -$grace_period)) ) {
        /* If it is not time for the normal polling cycle yet,
           check for devices that failed backup last cycle and
           attempt to back them up now. */

		/* Indicate start of poller */
		db_execute("REPLACE INTO settings (name, value) VALUES ('plugin_configstore_running', 1)");

        	foreach ($failed as $f) {
                	$device = array();
                	$device = db_fetch_row("SELECT * FROM plugin_configstore_devices WHERE id = " . $f['id']);
                	$result = plugin_configstore_download_config($device);
                	$devices_polled++;

                	/* Check for backup failure and update backup results */
                	if ($result) {
                	        db_execute("UPDATE plugin_configstore_devices SET backupresult='0', failedcount='0' WHERE id = " . $f['id']);
                	} else {
                	        $failedcount = db_fetch_cell("SELECT failedcount FROM plugin_configstore_devices WHERE id = " . $f['id']);
                	        $failedcount++;
                       	 	$fail_report[] = array ('hostname'      => $device['hostname'],
                                	                'ipaddress'     => $device['ipaddress'],
                                        	        'failcount'     => $failedcount);

                        	if ($failedcount >= 3) {
                                	db_execute("UPDATE plugin_configstore_devices SET backupresult='1', failedcount='0', enabled='' WHERE id = " . $f['id']); 
                        	} else {
                                	db_execute("UPDATE plugin_configstore_devices SET backupresult='1', failedcount='$failedcount' WHERE id = " . $f['id']);  
                        	}
                	}
        	}
		db_execute("REPLACE INTO settings (name, value) VALUES ('plugin_configstore_running', 0)");

		$success_count = $devices_polled - count($fail_report);
		cacti_log("$success_count devices backed up, which had failed during regular polling cycle.", true, "Configstore");

	}
}
