<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

include_once($config['base_path'] . '/plugins/configstore/include/logger.class.php');

global $log;
$log = new Logging();

function plugin_configstore_retention () {
	global $log, $config;
	$backuppath = read_config_option("plugin_configstore_backup_path");
	if (!is_dir($backuppath) || strlen($backuppath) < 2) {
		print "Backup Path is not set or is not a directory";
		exit;
	}

	$backup_config_days = read_config_option('plugin_configstore_retention');
	if ($backup_config_days < 1 || $backup_config_days > 365) {
		$backup_config_days = 30;
	}

	$backup_retention_time = time() - ($backup_config_days * 24 * 60 * 60);
	$backup_ids = array();
	$backups = db_fetch_assoc("SELECT id FROM plugin_configstore_backups WHERE btime < $backup_retention_time");
	foreach ($backups as $b) { array_push($backup_ids, $b['id']);  }
	$archive_results = plugin_configstore_archive_backups ($backup_ids);

	$archive_days = read_config_option('plugin_configstore_archive_retention');
        if ( ($archive_days < 1) || (!isset($archive_days)) ) {
                $archive_days = 365;
        }
        $archive_retention_time = time() - ($archive_days * 24 * 60 * 60);
	db_execute("DELETE FROM plugin_configstore_archive WHERE btime < $archive_retention_time");
}

function plugin_configstore_download_config ($device) {
        global $log, $config;
	$info = plugin_configstore_retrieve_account($device['id']);

	$backup_results = array();

	// Define backup file path
        $backuppath = read_config_option("plugin_configstore_backup_path");
        if (!is_dir($backuppath) || strlen($backuppath) < 2) {
		return false;
        } else {
		$device["backuppath"] = $backuppath;
	}

	// Define Backup server IP
        $server = read_config_option("plugin_configstore_backup_server");

        if (strlen($server) < 2) {
		return false;
        } else {
		$device["backup_server"] = $server;
	}

	// Update database with timestamp of last backup attempt
	db_execute('UPDATE plugin_configstore_devices SET lastattempt = ' . time() . ' WHERE id = ' . $device['id']);

	// Get backup script for device type
	$backup_script = db_fetch_cell("SELECT backup_script FROM plugin_configstore_device_types WHERE id = " . $device['devicetype']);
	$php_binary = trim(read_config_option("path_php_binary"));
	$backup_function = 0;

	// Index the backup script and attempt a backup
	if ( (isset($backup_script)) && (file_exists($config['base_path'] . '/plugins/configstore/lib/' . $backup_script)) ) {
		$cmd = $config['base_path'] . '/plugins/configstore/lib/' . $backup_script . " " . "index";
		exec("$php_binary $cmd", $output);
		$backup_function = implode("", $output);
		trim($backup_function);
		if ( (isset($backup_function)) && ($backup_function != "0") ) {
			include_once($config['base_path'] . '/plugins/configstore/lib/' . $backup_script);
			$backup_results = $backup_function($device);
		} else {
			return false;
		}
	} else {
                return false;
        }

	// Check results for the backup attempt and update database
	if (empty($backup_results)) {
		return false;
	} else {
		if ( (isset($backup_results['filename'])) && (isset($backup_results['device'])) ) {
			$myhostname = $backup_results['hostname'];

			db_execute("UPDATE plugin_configstore_devices SET hostname = '$myhostname' WHERE id = " . $backup_results['device']);
		} else {
			return false;
		}

		if ( (isset($backup_results['device'])) && (isset($backup_results['btime'])) && (isset($backup_results['directory'])) && (isset($backup_results['filename'])) && (isset($backup_results['config'])) && (isset($backup_results['lastchange'])) && (isset($backup_results['username'])) ) {
			$mybtime = $backup_results['btime'];
			$mydirectory = $backup_results['directory'];
			$myfilename = $backup_results['filename'];
			$myconfig = $backup_results['config'];
			$mylastchange = $backup_results['lastchange'];
			$myusername = $backup_results['username'];
			$mytime = time();

			db_execute('INSERT INTO plugin_configstore_backups (device, btime, directory, filename, config, lastchange, username) VALUES (' . $backup_results['device'] . ",'$mybtime','$mydirectory','$myfilename','$myconfig','$mylastchange','$myusername')");

                        db_execute("UPDATE plugin_configstore_devices SET lastbackup = '$mytime', lastchange = '$mylastchange', username = '$myusername' WHERE id = " . $backup_results['device']);

			return true;

		} else {
			return false;
		}
	}
}

function plugin_configstore_save_debug($device, $debug) {
        $debug = base64_encode($debug);
        echo "Saving Debug\n";
        db_execute("UPDATE plugin_configstore_devices SET debug = '$debug' WHERE id = " . $device['id']);
}

function plugin_configstore_save_error ($id, $error) {
	db_execute("UPDATE plugin_configstore_devices SET lasterror = '$error' WHERE id = $id");
}

function plugin_configstore_retrieve_account ($device) {
	global $log;
	$info = array();
	if (!isset($device)) {
		return false;
	} else {
		$info = db_fetch_row("SELECT plugin_configstore_accounts.* FROM plugin_configstore_accounts,plugin_configstore_devices WHERE plugin_configstore_accounts.id = plugin_configstore_devices.account AND plugin_configstore_devices.id = " . $device, FALSE);

		if (isset($info['password'])) {
			$info['password'] = plugin_configstore_decode($info['password']);
		}

		if (isset($info['enablepw'])) {
			$info['enablepw'] = plugin_configstore_decode($info['enablepw']);
		}

		if (isset($info['snmp_writestring'])) {
			$info['snmp_writestring'] = plugin_configstore_decode($info['snmp_writestring']);
		}

		if (isset($info['snmp_password'])) {
			$info['snmp_password'] = plugin_configstore_decode($info['snmp_password']);
		}

		if (isset($info['snmp_priv_passphrase'])) {
       		        $info['snmp_priv_passphrase'] = plugin_configstore_decode($info['snmp_priv_passphrase']);
       	 	}

		return $info;
	}
}

function plugin_configstore_decode ($info) {
	if (isset($info)) {
		$info = base64_decode($info);
		return $info;
	} else {
		return false;
	}
}

function plugin_configstore_encode ($info) {
	if (isset($info)) {
		$info = base64_encode($info);
		return $info;
	} else {
		return false;
	}
}

function plugin_configstore_archive_backups ($backups) {
	global $config, $log, $delete_results;

        $backuppath = read_config_option("plugin_configstore_backup_path");

	if ( (isset($backups)) && (isset($backuppath)) ) {
                foreach ($backups as $b) {
                        /* ================= input validation ================= */
                        input_validate_input_number($b);
                        /* ==================================================== */

			$fileinfo = array();
			$fileinfo = db_fetch_row("SELECT plugin_configstore_backups.*, plugin_configstore_devices.hostname, plugin_configstore_devices.ipaddress
					 FROM plugin_configstore_devices,plugin_configstore_backups
					 WHERE plugin_configstore_backups.device = plugin_configstore_devices.id AND plugin_configstore_backups.id=$b", FALSE);

                	if ( (isset($fileinfo['hostname'])) && (isset($fileinfo['ipaddress'])) && (isset($fileinfo['directory'])) && (isset($fileinfo['btime'])) && (isset($fileinfo['filename'])) && (isset($fileinfo['config'])) && (isset($fileinfo['lastchange'])) && (isset($fileinfo['username'])) ) {
	                        $mybtime = $fileinfo['btime'];
				$mydirectory = $fileinfo['directory'];
        	                $myfilename = $fileinfo['filename'];
				$myhostname = $fileinfo['hostname'];
				$myipaddress = $fileinfo['ipaddress'];
                	        $myconfig = $fileinfo['config'];
				$myconfig = mysql_escape_string($myconfig);
                       		$mylastchange = $fileinfo['lastchange'];
            		       	$myusername = $fileinfo['username'];
                        	$myatime = time();
				$sql = "INSERT INTO plugin_configstore_archive (device_hostname, device_ipaddress, btime, filename, config, lastchange, username, atime) VALUES ('$myhostname','$myipaddress','$mybtime','$myfilename','$myconfig','$mylastchange','$myusername', '$myatime')";

				if (!file_exists("$backuppath/$mydirectory/$myfilename")) {
					db_execute("$sql");
					db_execute("DELETE FROM plugin_configstore_backups WHERE id = $b");
				} else {
					db_execute("$sql");
					db_execute("DELETE FROM plugin_configstore_backups WHERE id = $b");
					$delete_results = unlink("$backuppath/$mydirectory/$myfilename");
				}

				$is_empty = is_empty_dir("$backuppath/$mydirectory");
				if ($is_empty) {
					rmdir("$backuppath/$mydirectory");
				} else { 
					/* Do nothing */
				}
			}
		}
	} else {
		return false;
	}
	return true;
}

function configstore_devices_filter() {
        global $item_rows, $log;

        $device_types = db_fetch_assoc("SELECT id, name FROM plugin_configstore_device_types ORDER BY name", false);
	$sites = db_fetch_assoc("SELECT id, site_id, name FROM plugin_configstore_sites ORDER BY site_id", false);

        ?>
        <script type="text/javascript">
        <!--
        function applyDevicesFilterChange(objForm) {
		strURL = '?site=' + objForm.site.value;
                strURL = strURL + '&device_status=' + objForm.device_status.value;
                strURL = strURL + '&backupresult=' + objForm.backupresult.value;
		strURL = strURL + '&device_type=' + objForm.device_type.value;
                strURL = strURL + '&rows=' + objForm.rows.value;
                strURL = strURL + '&filter=' + objForm.filter.value;
                document.location = strURL;
        }

        -->
        </script>
        <tr>
                <form name="form_configstore_devices">
                <td>
                        <table cellpadding="1" cellspacing="0">
                                <tr>
                                        <td width="1"></td>
					<td width="40">
                                                &nbsp;Site:&nbsp;
                                        </td>
                                        <td width="5">
                                                <select name="site" onChange="applyDevicesFilterChange(document.form_configstore_devices)">
                                                        <option value="-1"<?php if (get_request_var_request("site") == "-1") {?> selected<?php }?>>All</option>
                                                        <?php
                                                        foreach ($sites as $s) {
                                                                print "<option value='" . $s['id'] . "'"; if (get_request_var_request("site") == $s['id']) { print " selected"; } print ">" . $s['site_id'] . " - " . $s['name'] . "</option>\n";
                                                        }
                                                        ?>
                                                </select>
                                        </td>
                                        <td width="40">
                                                &nbsp;Status:&nbsp;
                                        </td>
                                        <td width="1">
                                                <select name="device_status" onChange="applyDevicesFilterChange(document.form_configstore_devices)">
							<option value="-1"<?php if (get_request_var_request("device_status") == "-1") {?> selected<?php }?>>All</option>
							<option value="0"<?php if (get_request_var_request("device_status") == "0") {?> selected<?php }?>>Enabled</option>
							<option value="1"<?php if (get_request_var_request("device_status") == "1") {?> selected<?php }?>>Disabled</option>
                                                </select>
                                        </td>
                                        <td width="40">
                                                &nbsp;Backup&nbsp;Result:&nbsp;
                                        </td>
                                        <td width="1">
                                                <select name="backupresult" onChange="applyDevicesFilterChange(document.form_configstore_devices)">
                                                        <option value="-1"<?php if (get_request_var_request("backupresult") == "-1") {?> selected<?php }?>>All</option>
                                                        <option value="0"<?php if (get_request_var_request("backupresult") == "0") {?> selected<?php }?>>Success</option>
                                                        <option value="1"<?php if (get_request_var_request("backupresult") == "1") {?> selected<?php }?>>Failed</option>
                                                </select>
                                        </td>
                                </tr>
                        </table>
                        <table cellpadding="1" cellspacing="0">
                                <tr>
<td width="40">
                                                &nbsp;&nbsp;Device&nbsp;Type:&nbsp;
                                        </td>
                                        <td width="1">
                                                <select name="device_type" onChange="applyDevicesFilterChange(document.form_configstore_devices)">
                                                        <option value="-1"<?php if (get_request_var_request("device_type") == "-1") {?> selected<?php }?>>All</option>
                                                        <?php
                                                        foreach ($device_types as $dt) {
                                                                print "<option value='" . $dt['id'] . "'"; if (get_request_var_request("device_type") == $dt['id']) { print " selected"; } print ">" . $dt['name'] . "</option>\n";
                                                        }
                                                        ?>
                                                </select>
                                        </td>
                                        <td width="40">
                                                &nbsp;Rows:&nbsp;
                                        </td>
                                        <td width="1">
                                                <select name="rows" onChange="applyDevicesFilterChange(document.form_configstore_devices)">
                                                        <option value="-1"<?php if (get_request_var_request("rows") == "-1") {?> selected<?php }?>>Default</option>
                                                        <?php
                                                        if (sizeof($item_rows) > 0) {
                                                                foreach ($item_rows as $key => $value) {
                                                                        print "<option value='" . $key . "'"; if (get_request_var_request("rows") == $key) { print " selected"; } print ">" . $value . "</option>\n";
                                                                }
                                                        }
                                                        ?>
                                                </select>
                                        </td>
                                        <td width="1"></td>
                                        <td width="40">
                                                &nbsp;Search:&nbsp;
                                        </td>
                                        <td width="1">
                                                <input type="text" name="filter" size="20" value="<?php print $_REQUEST["filter"];?>">
                                        </td>
                                        <td>
                                                &nbsp;<input type="submit" name="go_x" title="Submit Query" value="Go">
                                        </td>
                                        <td>
                                                &nbsp;<input type="submit" name="clear_x" title="Clear Filtered Results" value="Clear">
                                        </td>
                                </tr>
                        </table>
                </td>
                <input type='hidden' name='page' value='1'>
                </form>
        </tr>
        <?php
}

function configstore_generic_filter() {
        global $item_rows;

        ?>
        <script type="text/javascript">
        <!--
        function applyGenericFilterChange(objForm) {
                strURL = '?rows=' + objForm.rows.value;
                strURL = strURL + '&filter=' + objForm.filter.value;
                document.location = strURL;
        }

        -->
        </script>
        <tr>
                <form name="form_configstore_filter">
                <td>
                        <table cellpadding="1" cellspacing="0">
                                <tr>
                                        <td width="1"></td>

                                        <td nowrap style='white-space: nowrap;' width="40">
                                                &nbsp;Rows:&nbsp;
                                        </td>
                                        <td width="1">
                                                <select name="rows" onChange="applyGenericFilterChange(document.form_configstore_filter)">
                                                        <option value="-1"<?php if (get_request_var_request("rows") == "-1") {?> selected<?php }?>>Default</option>
                                                        <?php
                                                        if (sizeof($item_rows) > 0) {
                                                        foreach ($item_rows as $key => $value) {
                                                                print "<option value='" . $key . "'"; if (get_request_var_request("rows") == $key) { print " selected"; } print ">" . $value . "</option>\n";
                                                        }
                                                        }
                                                        ?>
                                                </select>
                                        </td>
					
					<td width="1"></td>
                                        <td width="40">
                                                &nbsp;Search:&nbsp;
                                        </td>
                                        <td width="1">
                                                <input type="text" name="filter" size="20" value="<?php print $_REQUEST["filter"];?>">
                                        </td>
                                        <td>
                                                &nbsp;<input type="submit" name="go_x" title="Submit Query" value="Go">
                                        </td>
                                        <td>
                                                &nbsp;<input type="submit" name="clear_x" title="Clear Filtered Results" value="Clear">
                                        </td>
                                </tr>
                        </table>
                </td>
                <input type='hidden' name='page' value='1'>
                </form>
        </tr>
        <?php
}

function is_empty_dir($dir)
{
    if (($files = @scandir($dir)) && count($files) <= 2) {
        return true;
    }
    return false;
}

function get_files_in_dir($directory) {
        $files = array();
        if ($handle = opendir($directory)) {
                while (false !== ($file = readdir($handle))) {
                        if ($file != "." && $file != "..") {
                                if (is_dir($directory. "/" . $file)) {
                                        $files = array_merge($files, get_files_in_dir($directory. "/" . $file));
                                        $ext = substr(strrchr($file, '.'), 1);
                                        if ($ext == "php") {
                                                $files[$file] = $file;
                                        }
                                } else {
                                        $ext = substr(strrchr($file, '.'), 1);
                                        if ($ext == "php") {
                                                $files[$file] = $file;
                                        }
                                }
                        }
                }
                closedir($handle);
        }
        return $files;
}

function Sec2Time($time){
        if(is_numeric($time)){
                $value = array( "years" => 0,
                                "days" => 0,
                                "hours" => 0,
                                "minutes" => 0,
                                "seconds" => 0,
                                );
        if($time >= 31556926){
                $value["years"] = floor($time/31556926);
                $time = ($time%31556926);
        }
        if($time >= 86400){
                $value["days"] = floor($time/86400);
                $time = ($time%86400);
        }
        if($time >= 3600){
                $value["hours"] = floor($time/3600);
                $time = ($time%3600);
        }
        if($time >= 60){
                $value["minutes"] = floor($time/60);
                $time = ($time%60);
        }
        $value["seconds"] = floor($time);
        return (array) $value;
        } else {
                return (bool) FALSE;
        }
}

function configstore_sendmail ($from, $namefrom, $to, $nameto, $subject, $message) {
	global $config, $log;

        if (defined('CACTI_BASE_PATH')) {
                $config["base_path"] = CACTI_BASE_PATH;
        }

        include_once($config["base_path"] . "/plugins/configstore/functions.php");
        include_once($config['base_path'] . '/plugins/configstore/include/logger.class.php');

        $log = new Logging();

	//SMTP + SERVER DETAILS
	/* * * * CONFIGURATION START * * * */
	$smtpServer = read_config_option('plugin_configstore_smtp_server');
	$port = "25";
	$timeout = "30";
	$smtpAuth = read_config_option('plugin_configstore_smtp_auth');

	if ( (isset($smtpAuth)) && ($smtpAuth == 'on') ) {
		$username = read_config_option('plugin_configstore_email_username');
		$password = read_config_option('plugin_configstore_email_password');
	}

	$localhost = "localhost";
	$newLine = "\r\n";
	/* * * * CONFIGURATION END * * * * */

	//Connect to the host on the specified port
	$smtpConnect = fsockopen($smtpServer, $port, $errno, $errstr, $timeout);
	$smtpResponse = fgets($smtpConnect, 515);
	
	if(empty($smtpConnect)) {
		return false;
	} else {
	}

	if ( (isset($smtpAuth)) && ($smtpAuth == 'on') ) {
		//Request Auth Login
		fputs($smtpConnect,"AUTH LOGIN" . $newLine);
		$smtpResponse = fgets($smtpConnect, 515);

		//Send username
		fputs($smtpConnect, base64_encode($username) . $newLine);
		$smtpResponse = fgets($smtpConnect, 515);

		//Send password
		fputs($smtpConnect, base64_encode($password) . $newLine);
		$smtpResponse = fgets($smtpConnect, 515);
	}

	//Say Hello to SMTP
	fputs($smtpConnect, "HELO $localhost" . $newLine);
	$smtpResponse = fgets($smtpConnect, 515);

	//Email From
	fputs($smtpConnect, "MAIL FROM: $from" . $newLine);
	$smtpResponse = fgets($smtpConnect, 515);

	//Email To
	fputs($smtpConnect, "RCPT TO: $to" . $newLine);
	$smtpResponse = fgets($smtpConnect, 515);

	//Start the Email
	fputs($smtpConnect, "DATA" . $newLine);
	$smtpResponse = fgets($smtpConnect, 515);

	//Construct Headers
        $headers = "To: $nameto <$to>" . $newLine;
        $headers .= "From: $namefrom <$from>" . $newLine;
	$headers .= "MIME-Version: 1.0" . $newLine;
	$headers .= "Content-type: text/html; charset=iso-8859-1" . $newLine;
	$headers .= "Content-Transfer-Encoding: 7bit;" . $newLine;

	//Compose the Email
	fputs($smtpConnect, "Subject: $subject\n$headers\n\n$message\n" . $newLine);

	//End the Email
	fputs($smtpConnect, "." . $newLine);
	$smtpResponse = fgets($smtpConnect, 515);
	//End the Email

	// Say Bye to SMTP
	fputs($smtpConnect,"QUIT" . $newLine);
	$smtpResponse = fgets($smtpConnect, 515);

	return true;
}
