<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

/**
 * Logging class:
 * - contains lopen and lwrite methods
 * - lwrite will write message to the log file
 * - first call of the lwrite will open log file implicitly
 * - message is written with the following format: hh:mm:ss (script name) message
 */
global $config;

// define log file
$log_file = $config['base_path'] . '/plugins/configstore/log/configstore.log';

class Logging{

	// define file pointer
	private $fp = null;

	// write message to the log file
	public function lwrite($message){
		// if file pointer doesn't exist, then open log file
		if (!$this->fp) $this->lopen();

		// define script name
		$script_name = pathinfo($_SERVER['PHP_SELF'], PATHINFO_FILENAME);

		// define current time
		$time = date('H:i:s');

		// write current time, script name and message to the log file
		fwrite($this->fp, "$time ($script_name) $message\n");
	}

	// open log file
	private function lopen(){
		// define log file path and name
		$lfile = $this->log_file;

		// define the current date (it will be appended to the log file name)
		$today = date('Y-m-d');

		// open log file for writing only; place the file pointer at the end of the file
		// if the file does not exist, attempt to create it.
		$this->fp = fopen($lfile . '_' . $today, 'a') or exit("Can't open $lfile!");
		@chmod ($lfile . '_' . $today, 0775);
	}
}
?>
