<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

# Allow indexing of the script to provide the function name to ConfigStore
if ( (isset($argv[1])) && (trim($argv[1]) === 'index') ) {
    echo "cisco_asa_get_config\n";
    exit;
}

include_once($config['base_path'] . '/plugins/configstore/functions.php');

global $config, $log;

include_once($config['base_path'] . '/plugins/configstore/include/logger.class.php');

$log = new Logging();

function cisco_asa_get_config ($device) { 
	global $log, $config;

	$info = plugin_configstore_retrieve_account($device['id']);
	$token = '0';

	if ( ($info['account_type'] == '1') || ($info['account_type'] == '2') || ($info['account_type'] == '3') ) {
		$token = '1';
		$backup_results = cisco_asa_snmp_get_config($device, $info);
		if ($backup_results) {
			return $backup_results;
		} else {
			return false;
		}	
	}
        if ($info['account_type'] == '4') {
		$token = '1';
                if(!file_exists($config['base_path'] . '/plugins/configstore/lib/cisco_asa-telnet.exp')) {
                       return false;
                } else {
			$backup_results = cisco_asa_term_get_config($device, $info);
			if ($backup_results) {
                                return $backup_results;
                        } else {
                                return false;
                        }
                }
        } 
	if ($info['account_type'] == '5') {
		$token = '1';
       		if(!file_exists($config['base_path'] . '/plugins/configstore/lib/cisco_asa-ssh.exp')) {
                       return false;
                } else {
			$backup_results = cisco_asa_term_get_config($device, $info);
			if ($backup_results) {
				return $backup_results;
			} else {
				return false;
			}
                }
	} 
	if ($info['backup_protocol'] == '3') {
		$token = '1';
		if(!file_exists($config['base_path'] . '/plugins/configstore/lib/cisco_asa-scpserver.exp')) {
			return false;
		} else {
			$backup_results = cisco_asa_scpserver_get_config($device, $info);
			if ($backup_results) {
				return $backup_results;
			} else {
				return false;
			}
		}
	}

	if ($token == '0') {
		return false;
	}
}

function cisco_asa_snmp_get_config($device, $info) {
	// Currently not supported in MIBs
	return false;
}

function cisco_asa_term_get_config ($device, $info) {
	global $log, $config;

	$backup_protocol = '';
	$scp_username = '';
	$scp_password = '';

	$expect = read_config_option('plugin_configstore_expect_path');
	if (!is_file($expect)) {
		return false;
	}

	if ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "1") ) {
		$backup_protocol = "tftp";
	} elseif ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "2") ) {
		// Currently not supported on ASAs
		return false;
	}

        $enablepw = "0";
        if ( (isset($info['enablepw'])) && (!empty($info['enablepw'])) ) {
                $enablepw = $info['enablepw'];
        }

        $backup_filename = $device['hostname'] . ".cfg";
        $backup_path = $device['backuppath'];
        $backup_server = $device['backup_server'];
	$cmd;

	if ( (isset($info['account_type'])) && ($info['account_type'] == '4') ) {
		$telnet = read_config_option('plugin_configstore_telnet_path');
		if (!is_file($telnet)) {
			return false;
		}

		if ( (isset($backup_protocol)) && ($backup_protocol == "tftp") ) {
			$cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/cisco_asa-telnet.exp " . $device['ipaddress'] . " " . $info['username'] . " " . $info['password'] . " " . $enablepw . " " .$backup_protocol . " " . $backup_server . " " . $backup_filename . " " . $telnet;
		}
	} elseif ( (isset($info['account_type'])) && ($info['account_type'] == '5') ) {
		$ssh = read_config_option('plugin_configstore_ssh_path');
		if (!is_file($ssh)) {
			return false;
		}

		if ( (isset($backup_protocol)) && ($backup_protocol == "tftp") ) {
                        $cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/cisco_asa-ssh.exp " . $device['ipaddress'] . " " . $info['username'] . " " . $info['password'] . " " . $enablepw . " " .$backup_protocol . " " . $backup_server . " " . $backup_filename . " " . $ssh;
		}
	} else {
		return false;
	}

	$success;
	$debug = "DEBUG:\r\r";
	exec($cmd, $result, $return_code);

	if ( (!isset($result)) || (sizeof($result) < 1) ) {
    	} else {
		foreach ($result as $r) {
			if (preg_match("/\bRETURN CODE: 0\b/", $r)) {
				$success = 0;
			} elseif (preg_match("/\bRETURN CODE: 1\b/", $r)) { 
				$success = 1;
			} elseif (preg_match("/\bRETURN CODE: 2\b/", $r)) {
				$success = 2;
			} else {
				$success = 3;
			}
		}
	}
	
	$debug = implode("\r", $result);
	if ($success == 0) {
		$backup_results = process_cisco_asa_backup($device, $debug);
		return $backup_results;
	} else {
		plugin_configstore_save_debug($device, $debug);
		return false;
	}
}

function cisco_asa_scpserver_get_config ($device, $info) {
        global $log, $config;

        $cmd = '';

        $expect = read_config_option('plugin_configstore_expect_path');
        if (!is_file($expect)) {
                return false;
        }
        if ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "3") ) {
                $scp = read_config_option('plugin_configstore_scp_path');
                if (!is_file($scp)) {
                        return false;
                }
                $backup_filename = $device['hostname'] . ".cfg";
                $backup_path = $config['base_path'] . "/" . $device['backuppath'];
                $scp_username = $info['username'];
                $scp_password = $info['password'];

                if ( (isset($scp_username)) && (isset($scp_password)) && (isset($backup_filename)) && (isset($backup_path)) ) {
                        if (!is_file($config['base_path'] . "/plugins/configstore/lib/cisco_asa-scpserver.exp")) {
                                return false;
                        } else {
                                $cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/cisco_asa-scpserver.exp " . $device['ipaddress'] . " " . $scp_username . " " . $scp_password . " " . $backup_path . " " . $backup_filename . " " . $scp;
                        }
                } else {
                        return false;
                }
        } else {
                return false;
        }
        $success = '';
        $debug = "DEBUG:\r\r";
        exec($cmd, $result, $return_code);

        if ( (!isset($result)) || (sizeof($result) < 1) ) {
        } else {
                foreach ($result as $r) {
                        if (preg_match("/\bRETURN CODE: 0\b/", $r)) {
                                $success = 0;
                        } elseif (preg_match("/\bRETURN CODE: 1\b/", $r)) {
                                $success = 1;
                        } elseif (preg_match("/\bRETURN CODE: 2\b/", $r)) {
                                $success = 2;
                        } else {
                                $success = 3;
                        }
                }
        }

        $debug = implode("\r", $result);
        if ($success == 0) {
                $backup_results = process_cisco_asa_backup($device, $debug);
                return $backup_results;
        } else {
		plugin_configstore_save_debug($device, $debug);
                return false;
        }
}

function process_cisco_asa_backup ($device, $debug) {
	global $log, $config;

	$backup_filename = $device['hostname'] . ".cfg";
	$backup_path = $device['backuppath'];
	$filename = $device['hostname'];
	$dir = $device['directory'];
	$hostname = $device['hostname'];

	$x = 0;
	$data = array();
	while ($x < 30) {
	        if (file_exists("$backup_path/$backup_filename")) {
	        	clearstatcache();
	                $file = fopen("$backup_path/$backup_filename", 'r');
	                if (filesize("$backup_path/$backup_filename") > 0) {
	                	$data = fread($file, filesize("$backup_path/$backup_filename"));
	                }
	                fclose($file);
			break;
	        }
	        sleep(1);
	        $x++;
	}

	if (!file_exists("$backup_path/$backup_filename")) {
		plugin_configstore_save_debug($device, $debug);
	        return false;
	} else {
		@unlink("$backup_path/$backup_filename");
	}

	$data = str_replace("\n", "\n\n", $data);
        $data = explode("\n\n", $data);

	if (count($data) < 10) {
		plugin_configstore_save_debug($device, $debug);
	        return false;
	}

	$lastchange = '';
	$lastuser = '';

	$version = explode("\r", $debug);
	foreach ($version as $v) {
		if (strpos($v, 'last modified by') !== FALSE) {
	        	$t = explode(' ', $v);
	        	if (isset($t[4])) {
				$tt = explode('.', $t[6]);
				$ti = $tt[0];
	        		$td = array($t[8], $t[9], $t[10], $t[11], $ti);
				$lastuser = $t[4];
	           	    	$td = implode(' ', $td);
	                	$lastchange = strtotime($td);
	        	}
		}
	}

	foreach ($data as $d) {
		if (substr($d, 0, 9) == 'hostname ') {
			$filename = trim(substr($d, 9));
		        if ($device['hostname'] != $filename) {
		        	$hostname = $filename;
		        }
		}
	}
	
	$backup_results = array();
	$data = implode("\n", $data);
        if (strlen($data) > 100) {
                if (!is_dir("$backup_path/$dir")) {
                        mkdir("$backup_path/$dir", 0777, true);
                }

		$lastbackup = time();
                $date = date("Y-m-d-Hi");
		$filename = "$filename-$date.cfg";
                $file = fopen("$backup_path/$dir/$filename", 'w');
                fwrite($file, $data);
                fclose($file);
                $data = mysql_escape_string($data);

                $t = time();
                if ($lastchange == '') {
                	$lastchange = 0;
		}
                $backup_results = array('device'	=> $device['id'], 
					'btime'		=> $t, 
					'directory'	=> $dir,
					'filename'	=> $filename, 
					'config'	=> $data,
					'lastchange'	=> $lastchange,
					'username'	=> $lastuser,
					'hostname'	=> $hostname,
					); 
        } else {
                plugin_configstore_save_debug($device, $debug);
                return false;
        }

	plugin_configstore_save_debug($device, $debug);
	return $backup_results;
}

?>
