<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

# Allow indexing of the script to provide the function name to ConfigStore
if ( (isset($argv[1])) && (trim($argv[1]) === 'index') ) {
    echo "foundry_get_config\n";
    exit;
}

include_once($config['base_path'] . '/plugins/configstore/functions.php');

global $config, $log;

include_once($config['base_path'] . '/plugins/configstore/include/logger.class.php');

$log = new Logging();

function foundry_get_config ($device) { 
	global $log, $config;

	$info = plugin_configstore_retrieve_account($device['id']);
	$match = 0;

	if ( ($info['account_type'] == '1') || ($info['account_type'] == '2') || ($info['account_type'] == '3') ) {
		$match = 1;
		$backup_results = foundry_snmp_get_config($device, $info);
		if ($backup_results) {
			return $backup_results;
		} else {
			return false;
		}	
	}
        if ($info['account_type'] == '4') {
                if(!file_exists($config['base_path'] . '/plugins/configstore/lib/foundry-telnet.exp')) {
                                return false;
                } else {
			$match = 1;
			$backup_results = foundry_term_get_config($device, $info);
			if ($backup_results) {
                                return $backup_results;
                        } else {
                                return false;
                        }
                }
        } 
	if ($info['account_type'] == '5') {
        	if(!file_exists($config['base_path'] . '/plugins/configstore/lib/foundry-ssh.exp')) {
                                return false;
                } else {
			$match = 1;
			$backup_results = foundry_term_get_config($device, $info);
			if ($backup_results) {
				return $backup_results;
			} else {
				return false;
			}
                }
       }
	if ($info['backup_protocol'] == '3') {
		if(!file_exists($config['base_path'] . '/plugins/configstore/lib/foundry-scpserver.exp')) {
			return false;
		} else {
			$match = 1;
			$backup_results = foundry_scpserver_get_config($device, $info);
			if ($backup_results) {
				return $backup_results;
			} else {
				return false;
			}
		}
	}

	if ($match < 1) {
		return false;
	}
}

function foundry_snmp_get_config($device, $info) {
	global $log, $config; 
	
	$backup_protocol = '';
        $scp_username = '';
        $scp_password = '';
        $backup_filename = $device['hostname'] . ".cfg";
        $backup_path = $device['backuppath'];
        $backup_server = $device['backup_server'];
        $setcmd = '';

        $snmpset = read_config_option('plugin_configstore_snmpset_path');
	$snmpwalk = read_config_option('path_snmpwalk');
	$snmpget = read_config_option('path_snmpget');

        if (!is_file($snmpset)) {
                return false;
        }

        if (!isset($info['snmp_version'])) {
                return false;
        }

        if (!isset($info['snmp_retries'])) {
                return false;
        }

        if (!isset($info['snmp_timeout'])) {
                return false;
        }

        if (!isset($info['snmp_port'])) {
                return false;
        }

        if ( ($info['snmp_version'] == '1') || ($info['snmp_version'] == '2c') ) {
                if (!isset($info['snmp_writestring'])) {
                        return false;
                }

		$setcmd = "$snmpset " . "-r " . $info['snmp_retries'] . " " . "-t " . $info['snmp_timeout'] . " " . "-Oqv " . "-v " . $info["snmp_version"] . " " . "-c " . $info['snmp_writestring'] . " " . $device['ipaddress'] . ":" . $info['snmp_port'];

		$walkcmd = "$snmpwalk " . "-r " . $info['snmp_retries'] . " " . "-t " . $info['snmp_timeout'] . " " . "-v " . $info["snmp_version"] . " " . "-c " . $info['snmp_writestring'] . " " . $device['ipaddress'] . ":" . $info['snmp_port'];

		$getcmd = "$snmpget " . "-r " . $info['snmp_retries'] . " " . "-t " . $info['snmp_timeout'] . " " . "-Oqv " . "-v " . $info["snmp_version"] . " " . "-c " . $info['snmp_writestring'] . " " . $device['ipaddress'] . ":" . $info['snmp_port'];

        } 
	
	if ($info['snmp_version'] == '3') {
        	if (!isset($info['snmp_username'])) {
                	return false;
        	}

        	if (!isset($info['snmp_password'])) {
                	return false;
        	}

        	if (!isset($info['snmp_auth_protocol'])) {
                	return false;
        	}

        	if (!isset($info['snmp_priv_protocol'])) {
                	return false;
        	}

        	if (!isset($info['snmp_priv_passphrase'])) {
                	return false;
        	}

		$setcmd = "$snmpset " . "-r " . $info['snmp_retries'] . " " . "-t " . $info['snmp_timeout'] . " " . "-Oqv " . "-v " . $info["snmp_version"] . " " . "-u " . $info['snmp_username'] . " " . "-a " . $info['snmp_auth_protocol'] . " " . "-A " . $info['snmp_password'] . " " . "-x " . $info['snmp_priv_protocol'] . " " . "-X " . $info['snmp_priv_passphrase'] . " " . "-l authPriv " . $device['ipaddress'] . ":" . $info['snmp_port'];

		$walkcmd = "$snmpwalk " . "-r " . $info['snmp_retries'] . " " . "-t " . $info['snmp_timeout'] . " " . "-v " . $info["snmp_version"] . " " . "-u " . $info['snmp_username'] . " " . "-a " . $info['snmp_auth_protocol'] . " " . "-A " . $info['snmp_password'] . " " . "-x " . $info['snmp_priv_protocol'] . " " . "-X " . $info['snmp_priv_passphrase'] . " " . "-l authPriv " . $device['ipaddress'] . ":" . $info['snmp_port'];

		$getcmd = "$snmpget " . "-r " . $info['snmp_retries'] . " " . "-t " . $info['snmp_timeout'] . " " . "-Oqv " . "-v " . $info["snmp_version"] . " " . "-u " . $info['snmp_username'] . " " . "-a " . $info['snmp_auth_protocol'] . " " . "-A " . $info['snmp_password'] . " " . "-x " . $info['snmp_priv_protocol'] . " " . "-X " . $info['snmp_priv_passphrase'] . " " . "-l authPriv " . $device['ipaddress'] . ":" . $info['snmp_port'];

	}

        $debug = "DEBUG:\r\r";

	if ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "1") ) {
                $backup_protocol = "1";
        } else {
                return false;
        }
        $snmp_cmds_tftp = array(
                /* Sets the tftp server address to the IP address of the Backup server */
                "$setcmd 1.3.6.1.4.1.1991.1.1.2.1.5.0 a" => "$backup_server",
                        
                /* Sets the config filename to your desired file name */
                "$setcmd 1.3.6.1.4.1.1991.1.1.2.1.8.0 s" => "$backup_filename",
                                
		/* Sets the mode to copy running config to TFTP server and starts transfer */
                "$setcmd 1.3.6.1.4.1.1991.1.1.2.1.9.0 i" => "22");

	$snmp_cmds = array();
	if ($backup_protocol == "1") {
		$snmp_cmds = $snmp_cmds_tftp;
	} else {
		return false;
	}

	$results = array();
	$success = 1;
	foreach ($snmp_cmds as $snmp_cmd=>$value) {
		$result = array();
		exec("$snmp_cmd $value", $result);
		$result_string = implode('', $result);
		$result_string = trim($result_string, '"');
		if ( (isset($result_string)) && ($result_string == $value) ) {
			$success_string = "SUCCESS";
			if (isset($snmp_cmd)) {
				if ( ($info['snmp_version'] == '1') || ($info['snmp_version'] == '2c') ) {
					$snmp_cmd = str_replace($info['snmp_writestring'], "<rwcommunity>", $snmp_cmd);
				}
				if ($info['snmp_version'] == '3') {
					$snmp_cmd = str_replace($info['snmp_password'], "<snmpAuthpass>", $snmp_cmd);
					$snmp_cmd = str_replace($info['snmp_priv_passphrase'], "<snmpPrivpass>", $snmp_cmd);
				}
			} else {
				$snmp_cmd = '';
			}
			array_push($results, "$snmp_cmd $value $success_string\r");
		} else {
			$success = 1;
			$success_string = "FAILED";
                        if (isset($snmp_cmd)) {
				if ( ($info['snmp_version'] == '1') || ($info['snmp_version'] == '2c') ) {
                                	$snmp_cmd = str_replace($info['snmp_writestring'], "<rwcommunity>", $snmp_cmd);
				}
				if ($info['snmp_version'] == '3') {
                                        $snmp_cmd = str_replace($info['snmp_password'], "<snmpAuthpass>", $snmp_cmd);
                                        $snmp_cmd = str_replace($info['snmp_priv_passphrase'], "<snmpPrivpass>", $snmp_cmd);
                                }
                        } else {
                                $snmp_cmd = '';
                        }
			array_push($results, "$snmp_cmd  $value  $success_string" . "\r" . $result_string);
			$debug = implode("\n", $results);
			plugin_configstore_save_debug($device, $debug);
			return false;
		}
	}

	$logs = array();
	exec("$walkcmd 1.3.6.1.4.1.1991.1.1.2.6.2.1.4", $logs);
	$logtimecmd = "$getcmd 1.3.6.1.4.1.1991.1.1.2.6.2.1.5";

	$lastchange = '';
	$username = '';
	$log_out = array();

	foreach ($logs as $le) {
		if (strpos($le, '= STRING:') !== FALSE) {
                	$l = explode("= STRING:", $le);
                	$index = trim($l[0]);
                	$index = explode(".", $index);
                	$i = $index[9];
                	exec("$logtimecmd.$i", $time_stamp);
                	$time_stamp = preg_replace("/[\"]/", "", $time_stamp);
                	$time_stamp = implode(" ", $time_stamp);
                	$time_stamp = explode(" ", $time_stamp);

                	$ts = $time_stamp[3] . " " .  $time_stamp[4] . " " .  $time_stamp[0] . ":I:";
                	$log_entry = trim($l[1]);
                	$log_entry = preg_replace("/[\"]/", "", $log_entry);
	
                	$entry = $ts . $log_entry;
                	array_push($log_out, $entry);
        	}
	}
	asort($log_out);
	foreach ($log_out as $lo) { array_push($results, $lo); }
	$debug = implode("\n", $results);
	
	$c=0;
	while($c < 5) {
		if (is_file("$backup_path/$backup_filename")) {
			$success = 0;
			break;
		} else {
			sleep(1);
		}
	}

        $debug = implode("\r", $results);
        if ($success == 0) {
                $backup_results = process_foundry_backup($device, $debug);
                return $backup_results;
        } else {
                return false;
        }
}

function foundry_term_get_config ($device, $info) {
	global $log, $config;

	$backup_protocol = '';

	$expect = read_config_option('plugin_configstore_expect_path');
	if (!is_file($expect)) {
		return false;
	}

	if ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "1") ) {
		$backup_protocol = "tftp";
	} else {
		return false;
	}

        $enablepw = "0";
        if ( (isset($info['enablepw'])) && (!empty($info['enablepw'])) ) {
                $enablepw = $info['enablepw'];
        }

        $backup_filename = $device['hostname'] . ".cfg";
        $backup_path = $device['backuppath'];
        $backup_server = $device['backup_server'];
	$cmd;

	if ( (isset($info['account_type'])) && ($info['account_type'] == '4') ) {
		$telnet = read_config_option('plugin_configstore_telnet_path');
		if (!is_file($telnet)) {
			return false;
		}

		if ( (isset($backup_protocol)) && ($backup_protocol == "tftp") ) {
			if (!is_file($config['base_path'] . "/plugins/configstore/lib/foundry-telnet.exp")) {
				return false;
			} else {
				$cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/foundry-telnet.exp " . $device['ipaddress'] . " " . $info['username'] . " " . $info['password'] . " " . $enablepw . " " .$backup_protocol . " " . $backup_server . " " . $backup_filename . " " . $telnet;
			}
		}

	} elseif ( (isset($info['account_type'])) && ($info['account_type'] == '5') ) {
		$ssh = read_config_option('plugin_configstore_ssh_path');
		if (!is_file($ssh)) {
			return false;
		}

		if ( (isset($backup_protocol)) && ($backup_protocol == "tftp") ) {
			if (!is_file($config['base_path'] . "/plugins/configstore/lib/foundry-ssh.exp")) {
				return false;
			} else {
                        	$cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/foundry-ssh.exp " . $device['ipaddress'] . " " . $info['username'] . " " . $info['password'] . " " . $enablepw . " " .$backup_protocol . " " . $backup_server . " " . $backup_filename . " " . $ssh;
			}
		}

	} else {
		return false;
	}

	$success;
	$debug = "DEBUG:\r\r";
	exec($cmd, $result, $return_code);

	if ( (!isset($result)) || (sizeof($result) < 1) ) {
    	} else {
		foreach ($result as $r) {
			if (preg_match("/\bRETURN CODE: 0\b/", $r)) {
				$success = 0;
			} elseif (preg_match("/\bRETURN CODE: 1\b/", $r)) { 
				$success = 1;
			} elseif (preg_match("/\bRETURN CODE: 2\b/", $r)) {
				$success = 2;
			} else {
				$success = 3;
			}
		}
	}
	
	$debug = implode("\r", $result);
	if ($success == 0) {
		$backup_results = process_foundry_backup($device, $debug);
		return $backup_results;
	} else {
		plugin_configstore_save_debug($device, $debug);
		return false;
	}
}

function foundry_scpserver_get_config ($device, $info) {
        global $log, $config;
	
	$cmd = '';

        $expect = read_config_option('plugin_configstore_expect_path');
        if (!is_file($expect)) {
                return false;
        }
        if ( (isset($info['backup_protocol'])) && ($info['backup_protocol'] == "3") ) {
		$scp = read_config_option('plugin_configstore_scp_path');
                if (!is_file($scp)) {
                        return false;
                }
		$backup_filename = $device['hostname'] . ".cfg";
		$backup_path = $config['base_path'] . "/" . $device['backuppath'];
		$scp_username = $info['username'];
		$scp_password = $info['password'];

		if ( (isset($scp_username)) && (isset($scp_password)) && (isset($backup_filename)) && (isset($backup_path)) ) {
			if (!is_file($config['base_path'] . "/plugins/configstore/lib/foundry-scpserver.exp")) {
				return false;
			} else {		
				$cmd = "$expect " .  $config['base_path'] . "/plugins/configstore/lib/foundry-scpserver.exp " . $device['ipaddress'] . " " . $scp_username . " " . $scp_password . " " . $backup_path . " " . $backup_filename . " " . $scp;
			}
		} else {
			return false;
		}
        } else {
                return false;
        }

        $success = '';
        $debug = "DEBUG:\r\r";
        exec($cmd, $result, $return_code);

        if ( (!isset($result)) || (sizeof($result) < 1) ) {
        } else {
                foreach ($result as $r) {
                        if (preg_match("/\bRETURN CODE: 0\b/", $r)) {
                                $success = 0;
                        } elseif (preg_match("/\bRETURN CODE: 1\b/", $r)) {
                                $success = 1;
                        } elseif (preg_match("/\bRETURN CODE: 2\b/", $r)) {
                                $success = 2;
                        } else {
                                $success = 3;
                        }
                }
        }

        $debug = implode("\r", $result);
        if ($success == 0) {
                $backup_results = process_foundry_backup($device, $debug);
                return $backup_results;
        } else {
		plugin_configstore_save_debug($device, $debug);
                return false;
        }
}

function process_foundry_backup ($device, $debug) {
	global $log, $config;

	$backup_filename = $device['hostname'] . ".cfg";
	$backup_path = $device['backuppath'];
	$filename = $device['hostname'];
	$dir = $device['directory'];
	$hostname = $device['hostname'];

	$x = 0;
	$data = array();
	while ($x < 30) {
	        if (file_exists("$backup_path/$backup_filename")) {
	        	clearstatcache();
	                $file = fopen("$backup_path/$backup_filename", 'r');
	                if (filesize("$backup_path/$backup_filename") > 0) {
	                	$data = fread($file, filesize("$backup_path/$backup_filename"));
	                }
	                fclose($file);
			break;
	        }
	        sleep(1);
	        $x++;
	}

	if (!file_exists("$backup_path/$backup_filename")) {
		plugin_configstore_save_debug($device, $debug);
	        return false;
	} else {
		@unlink("$backup_path/$backup_filename");
	}

        $data = str_replace("\n", "\n\n", $data);
        $data = explode("\n\n", $data);

	if (count($data) < 10) {
		plugin_configstore_save_debug($device, $debug);
	        return false;
	}

	$lastchange = '';
	$lastuser = '';

	foreach ($data as $d) {
		if (substr($d, 0, 9) == 'hostname ') {
			$filename = trim(substr($d, 9));
		        if ($device['hostname'] != $filename) {
		        	$hostname = $filename;
		        }
		}

	}
	
	$debug = explode("\r", $debug);
	$out_array = array();
	foreach ($debug as $ug) {
		if (strpos($ug, '-config was changed by') !== FALSE) {
                	$u = preg_split("/:I:/", $ug);
                        $lastchange = trim($u[0]);
                        $username = preg_split("/\s+/", $u[1]);
                        $username = $username[4];
                        $out_array[] = array ('lastchange' => $lastchange, 'username' => $username);
                }
	}
	if (!empty($out_array)) {
		$out_array = array_slice($out_array, 0, 1);
	}

	$uptime = 0;
	foreach ($debug as $ug) {
		if (strpos($ug, ' uptime is ') !== FALSE) {
		        $up = trim(substr($ug, strpos($ug, ' uptime is ') + 11));
		        $up = explode(' ', $up);
		        $lastu = 0;
		        foreach ($up as $u) {
		                if ( (trim($u) == "years") || (trim($u) == "years") ) {
		                        $uptime += ($lastu * 31449600);
		                }

		                if ( (trim($u) == "month") || (trim($u) == "months") ) {
		                        $uptime += ($lastu * 2419200);
		                }

		                if ( (trim($u) == "week") || (trim($u) == "weeks") ) {
		                        $uptime += ($lastu * 604800);
		                }
		
		                if ( (trim($u) == "day") || (trim($u) == "days") ) {
		                        $uptime += ($lastu * 86400);
		                }

		                if ( (trim($u) == "hour") || (trim($u) == "hours") ) {
		                        $uptime += ($lastu * 3600);
		                }

		                if ( (trim($u) == "minute") || (trim($u) == "minutes") ) {
		                        $uptime += ($lastu * 60);
		                }

		                if ( (trim($u) == "second") || (trim($u) == "seconds") ) {
		                        $uptime += $lastu;
		                }

		                $lastu = $u;
		        }
		}
	}

	foreach ($out_array as $o) {
		$lastchange = $o['lastchange'];

		if (preg_match("/[0-9]{2}[d][0-9]{2}[h][0-9]{2}[m][0-9]{2}[s]/i", $lastchange)) {
		        $time_frame = preg_replace("/[A-z]/", " ", $lastchange);
		        $time_array = explode(" ", $time_frame);

		        $days = $time_array[0];
		        $hours = $time_array[1];
		        $minutes = $time_array[2];
		        $seconds = $time_array[3];

		        $lastchange = ( ($days * 86400) + ($hours * 3600) + ($minutes * 60) + $seconds);
		        $past_time = (time() - $uptime);
		        $lastchange = ($past_time + $lastchange);
		} else {
		        $lastchange = strtotime($lastchange);
		}

		$lastuser = $o['username'];
	}
	
	$backup_results = array();
	$data = implode("\n", $data);
	$debug = implode("\n", $debug);
        if (strlen($data) > 100) {
                if (!is_dir("$backup_path/$dir")) {
                        mkdir("$backup_path/$dir", 0777, true);
                }

		$lastbackup = time();
                $date = date("Y-m-d-Hi");
		$filename = "$filename-$date.cfg";
                $file = fopen($config['base_path'] . "/$backup_path/$dir/$filename", 'w');
                fwrite($file, $data);
                fclose($file);
                $data = mysql_escape_string($data);

                $t = time();
                if ($lastchange == '') {
                	$lastchange = 0;
		}
                $backup_results = array('device'	=> $device['id'], 
					'btime'		=> $t, 
					'directory'	=> $dir,
					'filename'	=> $filename, 
					'config'	=> $data,
					'lastchange'	=> $lastchange,
					'username'	=> $lastuser,
					'hostname'	=> $hostname,
					); 
        } else {
                plugin_configstore_save_debug($device, $debug);
                return false;
        }

	plugin_configstore_save_debug($device, $debug);
	return $backup_results;
}

?>
