<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

function plugin_configstore_install() {
	api_plugin_register_hook('configstore', 'top_header_tabs', 'configstore_show_tab', "setup.php");
	api_plugin_register_hook('configstore', 'top_graph_header_tabs', 'configstore_show_tab', "setup.php");
        api_plugin_register_hook('configstore', 'config_arrays', 'configstore_config_arrays', "setup.php");
        api_plugin_register_hook('configstore', 'draw_navigation_text', 'configstore_draw_navigation_text', "setup.php");
        api_plugin_register_hook('configstore', 'config_settings', 'configstore_config_settings', "setup.php");
        api_plugin_register_hook('configstore', 'poller_bottom', 'configstore_poller_bottom', "setup.php");
        api_plugin_register_hook('configstore', 'page_head', 'configstore_page_head', "setup.php");

	plugin_configstore_db_install ();
}

function plugin_configstore_uninstall () {
        if (configstore_db_table_exists("plugin_configstore_accounts")) {
                db_execute("DROP TABLE `plugin_configstore_accounts`");
        }

        if (configstore_db_table_exists("plugin_configstore_backups")) {
                db_execute("DROP TABLE `plugin_configstore_backups`");
        }

        if (configstore_db_table_exists("plugin_configstore_devices")) {
                db_execute("DROP TABLE `plugin_configstore_devices`");
        }
	
	if (configstore_db_table_exists("plugin_configstore_archive")) {
		db_execute("DROP TABLE `plugin_configstore_archive`");
	}

	if (configstore_db_table_exists("plugin_configstore_templates")) {
		db_execute("DROP TABLE `plugin_configstore_templates`");
	}

	if (configstore_db_table_exists("plugin_configstore_device_types")) {
		db_execute("DROP TABLE `plugin_configstore_device_types`");
	}

	if (configstore_db_table_exists("plugin_configstore_sites")) {
		db_execute("DROP TABLE `plugin_configstore_sites`");
	}
}

function plugin_configstore_check_config () {
        /* Here we will check to ensure everything is configured */
        configstore_check_upgrade();
        return true;
}

function plugin_configstore_upgrade () {
        /* Here we will upgrade to the newest version */
        configstore_check_upgrade();
        return false;
}

function plugin_configstore_version () {
        return configstore_version();
}

function configstore_check_upgrade () {
        global $config;

        if (defined('CACTI_BASE_PATH')) {
                $config["base_path"] = CACTI_BASE_PATH;
        }
        include_once($config["base_path"] . "/plugins/configstore/functions.php");

        $files = array('index.php', 'plugins.php', 'configstore_devices.php');
        if (isset($_SERVER['PHP_SELF']) && !in_array(basename($_SERVER['PHP_SELF']), $files)) {
                return;
        }

        $current = plugin_configstore_version();
        $current = $current['version'];

        $old = db_fetch_row("SELECT * FROM plugin_config WHERE directory='configstore'");
        if (!sizeof($old) || $current != $old["version"]) {
                /* if the plugin is installed and/or active */
                if (!sizeof($old) || $old["status"] == 1 || $old["status"] == 4) {
                        /* re-register the hooks */
                        plugin_configstore_install();
                        if (api_plugin_is_enabled('configstore')) {
                                # may sound ridiculous, but enables new hooks
                                api_plugin_enable_hooks('configstore');
                        }

                        /* perform a database upgrade */
                        configstore_database_upgrade();
                }

                /* update the plugin information */
                $info = plugin_configstore_version();
                $id   = db_fetch_cell("SELECT id FROM plugin_config WHERE directory='configstore'");

                db_execute("UPDATE plugin_config
                        SET name='" . $info["longname"] . "',
                        author='"   . $info["author"]   . "',
                        webpage='"  . $info["homepage"] . "',
                        version='"  . $info["version"]  . "'
                        WHERE id='$id'");
        }
}

function configstore_db_table_exists($table) {
        return sizeof(db_fetch_assoc("SHOW TABLES LIKE '$table'"));
}

function configstore_db_column_exists($table, $column) {
        $found = false;

        if (configstore_db_table_exists($table)) {
                $columns  = db_fetch_assoc("SHOW COLUMNS FROM $table");
                if (sizeof($columns)) {
                	foreach($columns as $row) {
                        	if ($row["Field"] == $column) {
                                	$found = true;
                                	break;
                        	}
                	}
                }
        }
        return $found;
}

function configstore_db_key_exists($table, $index) {
        $found = false;

        if (configstore_db_table_exists($table)) {
                $keys  = db_fetch_assoc("SHOW INDEXES FROM $table");
                if (sizeof($keys)) {
	                foreach($keys as $key) {
        	                if ($key["Key_name"] == $index) {
                	                $found = true;
                        	        break;
                        	}
                	}
                }
        }
        return $found;
}

function configstore_add_column($table, $column, $syntax) {
        if (!configstore_db_column_exists($table, $column)) {
                db_execute($syntax);
        }
}

function configstore_database_upgrade () {
	configstore_add_column("plugin_configstore_devices", "site", "ALTER TABLE `plugin_configstore_devices` ADD COLUMN `site` int(12) unsigned NOT NULL default '0' AFTER `hostname`");		
}
	

function configstore_check_dependencies() {
        global $plugins, $config;

        return true;
}

function plugin_configstore_db_install () {
        if (!configstore_db_table_exists("plugin_configstore_accounts")) {
                db_execute("CREATE TABLE `plugin_configstore_accounts` (
  			`id` int(12) NOT NULL auto_increment,
			`name` varchar(64) NOT NULL,
			`account_type` tinyint(11) unsigned NOT NULL default '0',
			`backup_protocol` tinyint(11) unsigned NOT NULL default '1',
                        `snmp_version` varchar(100) NOT NULL default '',
                        `snmp_writestring` varchar(100) NOT NULL,
                        `snmp_port` int(10) NOT NULL default '161',
                        `snmp_timeout` int(10) unsigned NOT NULL default '500',
                        `snmp_retries` tinyint(11) unsigned NOT NULL default '3',
                        `snmp_username` varchar(50) default NULL,
                        `snmp_password` varchar(50) default NULL,
                        `snmp_auth_protocol` char(5) default '',
                        `snmp_priv_passphrase` varchar(200) default '',
                        `snmp_priv_protocol` char(6) default '',
                        `snmp_context` varchar(64) default '',
        	        `username` varchar(64) NOT NULL,
                	`password` varchar(256) default NULL,
               		`enablepw` varchar(256) default NULL,
                	`private_key_path` varchar(128) default '',
  			PRIMARY KEY (`id`)) ENGINE=MyISAM;");
	}

        if (!configstore_db_table_exists("plugin_configstore_backups")) {
		db_execute("CREATE TABLE `plugin_configstore_backups` (
  			`id` int(12) NOT NULL auto_increment,
  			`btime` int(18) NOT NULL,
  			`device` int(12) NOT NULL,
  			`directory` varchar(255) NOT NULL,
  			`filename` varchar(255) NOT NULL,
  			`config` longtext NOT NULL,
  			`lastchange` int(24) NOT NULL,
  			`username` varchar(128) NOT NULL,
  			PRIMARY KEY  (`id`),
  			KEY btime (`btime`),
  			KEY `device` (`device`),
  			KEY `directory` (`directory`),
  			KEY `lastchange` (`lastchange`)) ENGINE=MyISAM;");
	}

        if (!configstore_db_table_exists("plugin_configstore_archive")) {
                db_execute("CREATE TABLE `plugin_configstore_archive` (
                        `id` int(12) NOT NULL auto_increment,
                        `btime` int(18) NOT NULL,
                        `device_hostname` varchar(255) NOT NULL,
			`device_ipaddress` varchar(128) NOT NULL,
                        `filename` varchar(255) NOT NULL,
                        `config` longtext NOT NULL,
                        `lastchange` int(24) NOT NULL,
                        `username` varchar(128) NOT NULL,
			`atime` int(18) NOT NULL,
                        PRIMARY KEY  (`id`),
                        KEY btime (`btime`),
			KEY atime (`atime`),
                        KEY `device` (`device_hostname`),
                        KEY `lastchange` (`lastchange`)) ENGINE=MyISAM;");
        }

        if (!configstore_db_table_exists("plugin_configstore_devices")) {
		db_execute("CREATE TABLE `plugin_configstore_devices` (
  			`id` int(12) NOT NULL auto_increment,
  			`enabled` varchar(2) NOT NULL default 'on',
  			`ipaddress` varchar(128) NOT NULL,
  			`hostname` varchar(255) NOT NULL,
  			`directory` varchar(255) NOT NULL,
  			`account` int(12) NOT NULL,
  			`lastchange` int(24) NOT NULL,
			`username` varchar(128) NOT NULL,
  			`schedule` int(6) NOT NULL default 0,
  			`lastbackup` int(18) NOT NULL,
  			`lastattempt` int(18) NOT NULL,
			`backupresult` int(1) NOT NULL,
			`failedcount` int(1) NOT NULL,
  			`devicetype` int(12) NOT NULL,
  			`debug` text NOT NULL,
			`site` int(12) NOT NULL,
  			PRIMARY KEY  (`id`),
			KEY `backupresult` (`backupresult`),
  			KEY `enabled` (`enabled`),
			KEY `site` (`site`),
  			KEY `schedule` (`schedule`),
  			KEY `ipaddress` (`ipaddress`),
  			KEY `account` (`account`),
  			KEY `lastbackup` (`lastbackup`),
  			KEY `lastchange` (`lastchange`),
  			KEY `lastattempt` (`lastattempt`),
  			KEY `devicetype` (`devicetype`)) ENGINE=MyISAM;");
	}

	if (!configstore_db_table_exists("plugin_configstore_templates")) {
		db_execute("CREATE TABLE `plugin_configstore_templates` (
			`id` int(12) NOT NULL auto_increment,
			`name` varchar(128) NOT NULL,
			`filename` varchar(255) NOT NULL,
			`lastchange` int(24) NOT NULL,
			`username` varchar(128) NOT NULL,
			`config` longtext NOT NULL,
			PRIMARY KEY (`id`),
			KEY `name` (`name`),
			KEY `filename` (`filename`),
			KEY `lastchange` (`lastchange`),
			KEY `username` (`username`)) ENGINE=MyISAM;");
	}

        if (!configstore_db_table_exists("plugin_configstore_device_types")) {
                db_execute("CREATE TABLE `plugin_configstore_device_types` (
                        `id` int(12) NOT NULL auto_increment,
                        `name` varchar(64) NOT NULL,
                        `vendor` varchar(64) NOT NULL,
                        `models` varchar(128) NOT NULL default '',
                        `backup_script` varchar(128) NOT NULL default '',
                        PRIMARY KEY (`id`)) ENGINE=MyISAM;");

                db_execute("INSERT INTO `plugin_configstore_device_types` (
                        `name`, `vendor`, `models`, `backup_script`) VALUES (
                        'Cisco-IOS', 'Cisco Systems', 'IOS routers and switches', 'cisco-ios.php')");               

		db_execute("INSERT INTO `plugin_configstore_device_types` (
			`name`, `vendor`, `models`, `backup_script`) VALUES (
			'Foundry', 'Foundry Networks', 'Foundry, now Brocade Ethernet switches and routers', 'foundry.php')");

		db_execute("INSERT INTO `plugin_configstore_device_types` (
                        `name`, `vendor`, `models`, `backup_script`) VALUES (
                        'Cisco-ASA', 'Cisco Systems', 'Cisco ASA Firewalls', 'cisco-asa.php')");
        }

	if (!configstore_db_table_exists("plugin_configstore_sites")) {
		db_execute("CREATE TABLE `plugin_configstore_sites` (
			`id` int(12) NOT NULL auto_increment,
			`site_id` varchar(12) NOT NULL,
			`name` varchar(128) NOT NULL,
			`address` varchar(128) NOT NULL,
			`suite` varchar(12) NOT NULL,
			`city` varchar(128) NOT NULL,
			`state` varchar(128) NOT NULL,
			`zip` varchar(12) NOT NULL,
			`facility_contact_name` varchar(128) NOT NULL default '',
			`facility_contact_number` varchar(64) NOT NULL default '',
			`facility_contact_email` varchar(128) NOT NULL default '',
			`technical_contact_name` varchar(128) NOT NULL default '',
			`technical_contact_number` varchar(64) NOT NULL default '',
			`technical_contact_email` varchar(128) NOT NULL default '',
			`site_information` varchar(255) NOT NULL default '',
			PRIMARY KEY (`id`),
			KEY `site_id` (`site_id`),
			KEY `name` (`name`)) ENGINE=MyISAM;");
	}
}

function configstore_version () {
        return array(
                'name'      => 'ConfigStore',
                'version'   => '0.7',
                'longname'  => 'ConfigStore',
                'author'    => 'Chadd J. Christiansen',
                'homepage'  => 'http://cacti.net',
                'email'     => 'chadd.christiansen@gmail.com',
                'url'       => 'http://cactiusers.org/cacti/versions.php'
        );
}

function configstore_page_head () {
	global $config;

	if (strpos($_SERVER['PHP_SELF'], 'configstore_compare.php')) {
		print '<link rel="stylesheet" type="text/css" href="' . $config['url_path'] . "plugins/configstore/diff.css\">\n";
	}
}

function configstore_poller_bottom () {
	global $config;

        if (defined('CACTI_BASE_PATH')) {
                $config["base_path"] = CACTI_BASE_PATH;
        }
	include_once($config["base_path"] . "/lib/poller.php");

	$running = read_config_option("plugin_configstore_running");
	if ($running == "1") {
		return;
	}

	$command_string = trim(read_config_option("path_php_binary"));
	$extra_args = " -q " . $config["base_path"] . "/plugins/configstore/configstore_poller.php";
	exec_background($command_string, $extra_args);
}

function configstore_config_settings () {
	global $tabs, $settings, $config, $log;

        if (defined('CACTI_BASE_PATH')) {
                $config["base_path"] = CACTI_BASE_PATH;
        }

        include_once($config['base_path'] . '/plugins/configstore/functions.php');
	include_once($config['base_path'] . '/plugins/configstore/include/logger.class.php');

	$log = new Logging();


	$tabs["configstore"] = "ConfigStore";

	$settings["configstore"] = array(
		"plugin_configstore_header_tools" => array(
			"friendly_name" => "Required Tool Paths",
			"method" => "spacer",
			),
		"plugin_configstore_snmpset_path" => array(
			"friendly_name" => "snmpset Binary Path",
			"description" => "The path to your snmpset binary.",
			"method" => "filepath",
			"max_length" => 255,
			"default" => "/usr/bin/snmpset"
			),
                "plugin_configstore_expect_path" => array(
                        "friendly_name" => "expect Binary Path",
                        "description" => "The path to your expect binary.",
                        "method" => "filepath",
                        "max_length" => 255,
                        "default" => "/usr/bin/expect"
                        ),
                "plugin_configstore_ssh_path" => array(
                        "friendly_name" => "SSH Binary Path",
                        "description" => "The path to your SSH binary.",
                        "method" => "filepath",
                        "max_length" => 255,
                        "default" => "/usr/bin/ssh"
                        ),
                "plugin_configstore_scp_path" => array(
                        "friendly_name" => "SCP Binary Path",
                        "description" => "The path to your SCP binary.",
                        "method" => "filepath",
                        "max_length" => 255,
                        "default" => "/usr/bin/scp"
                        ),
                "plugin_configstore_telnet_path" => array(
                        "friendly_name" => "telnet Binary Path",
                        "description" => "The path to your Telnet binary.",
                        "method" => "filepath",
                        "max_length" => 255,
                        "default" => "/usr/bin/telnet"
                        ),
		"plugin_configstore_header_general" => array(
			"friendly_name" => "General Settings",
			"method" => "spacer",
			),
		"plugin_configstore_backup_server" => array(
			"friendly_name" => "Backup Server IP Address",
			"description" => "Must be an IP address pointing to your Cacti server.",
			"method" => "textbox",
			"max_length" => 255,
			),
		"plugin_configstore_backup_path" => array(
			"friendly_name" => "Backup Directory Path",
			"description" => "The path to where your Configs will be backed up, it must be the path that the local TFTP Server, or SCP user writes to.",
			"method" => "dirpath",
			"max_length" => 255,
			'default' => 'plugins/configstore/backups'
			),
	        "plugin_configstore_scp_username" => array(
                        "friendly_name" => "SCP Username",
                        "description" => "Username used to authenticate when sending backups to the above server via SCP.",
                        "method" => "textbox",
                        "size" => 40,
                        "max_length" => 255,
                        "default" => ''
                        ),
                "plugin_configstore_scp_password" => array(
                        "friendly_name" => "SCP Password",
                        "description" => "Password used to authenticate when sending backups to the above server via SCP.",
                        "method" => "textbox_password",
                        "size" => 40,
                        "max_length" => 255,
                        "default" => ''
                        ),	
                "plugin_configstore_retention" => array(
                        "friendly_name" => "Retention Period",
                        "description" => "The number of days to retain old backups.",
                        "method" => "textbox",
                        "size" => 5,
                        "max_length" => 3,
                        "default" => '30'
                        ),
                "plugin_configstore_backup_time" => array(
                        "friendly_name" => "Start Time for Backups",
                        "description" => "When would you backups to take place. A good example would be 12:00AM.",
                        "default" => "1:00AM",
                        "method" => "textbox",
                        "max_length" => "10",
                        "size" => "8"
                        ),
                "plugin_configstore_archive_retention" => array(
                        "friendly_name" => "Archive Retention Period",
                        "description" => "The number of days to retain archived backups (recyle bin).",
                        "method" => "textbox",
                        "size" => 5,
                        "max_length" => 3,
                        "default" => '365'
                        ),
	        "plugin_configstore_email_header" => array(
                        "friendly_name" => "Email Settings",
                        "method" => "spacer",
                        ),
                "plugin_configstore_smtp_server" => array(
                        "friendly_name" => "SMTP Server",
                        "description" => "SMTP server to send email from.",
                        "method" => "textbox",
                        "size" => 40,
                        "max_length" => 255,
                        "default" => ''
                        ),
		"plugin_configstore_smtp_auth" => array(
                        "friendly_name" => "SMTP Server Authentication Required",
                        "description" => "Check the box if you server requires authentications, and fill out the username and password fields approprietly",
                        "method" => "checkbox",
                        "default" => ''
                        ),
                "plugin_configstore_email_username" => array(
                        "friendly_name" => "SMTP Username",
                        "description" => "Username used to authenticate before sending email.",
                        "method" => "textbox",
                        "size" => 40,
                        "max_length" => 255,
                        "default" => ''
                        ),
                "plugin_configstore_email_password" => array(
                        "friendly_name" => "SMTP Password",
                        "description" => "Password used to authenticate before sending email.",
                        "method" => "textbox_password",
                        "size" => 40,
                        "max_length" => 255,
                        "default" => ''
                        ),
		"plugin_configstore_email_to" => array(
			"friendly_name" => "Email Address",
			"description" => "Email address to send the backup email to.  Comma deliminate any extra email addresses.",
			"method" => "textbox",
			"size" => 40,
			"max_length" => 255,
			"default" => ''
			),
		"plugin_configstore_email_name_to" => array(
                        "friendly_name" => "Name email is going to",
                        "description" => "Name to display in the email header as the recipient.",
                        "method" => "textbox",
                        "size" => 40,
                        "max_length" => 255,
                        "default" => ''
                        ),
		"plugin_configstore_email_from" => array(
			"friendly_name" => "From Address",
			"description" => "Email address the nightly backup will be sent from.",
			"method" => "textbox",
			"size" => 40,
			"max_length" => 255,
			"default" => ''
			),
                "plugin_configstore_email_name_from" => array(
                        "friendly_name" => "Name email is coming from",
                        "description" => "Name to display in the email header as the sender.",
                        "method" => "textbox",
                        "size" => 40,
                        "max_length" => 255,
                        "default" => ''
                        ),
                "plugin_configstore_email_subject" => array(
                        "friendly_name" => "Subject of the email",
                        "description" => "Subject of the email being sent for backups.",
                        "method" => "textbox",
                        "size" => 40,
                        "max_length" => 255,
                        "default" => ''
                        ),
		);

        $ts = array();
        foreach ($settings['path'] as $t => $ta) {
                $ts[$t] = $ta;
        }
        $settings['path']=$ts;
        configstore_check_upgrade();
}

function configstore_draw_navigation_text ($nav) {
	$nav["configstore_devices.php:"] = array("title" => "Configstore Devices", "mapping" => "index.php:", "url" => "configstore_devices.php", "level" => "1");
	$nav["configstore_devices.php:edit"] = array("title" => "Configstore Devices", "mapping" => "index.php:", "url" => "configstore_devices.php", "level" => "1");
	$nav["configstore_devices.php:actions"] = array("title" => "Configstore Devices", "mapping" => "index.php:", "url" => "configstore_devices.php", "level" => "1");
	$nav["configstore_devices.php:viewconfig"] = array("title" => "Configstore Devices", "mapping" => "index.php:", "url" => "configstore_devices.php", "level" => "1");
	$nav["configstore_devices.php:viewdebug"] = array("title" => "Configstore Devices", "mapping" => "index.php:", "url" => "configstore_devices.php", "level" => "1");

	$nav["configstore_backups.php:"] = array("title" => "Configstore Device Backups", "mapping" => "index.php:", "url" => "configstore_backups.php", "level" => "1");
	$nav["configstore_backups.php:edit"] = array("title" => "Configstore Device Backups", "mapping" => "index.php:", "url" => "configstore_backups.php", "level" => "1");
	$nav["configstore_backups.php:actions"] = array("title" => "Configstore Device Backups", "mapping" => "index.php:", "url" => "configstore_backups.php", "level" => "1");
	$nav["configstore_backups.php:viewconfig"] = array("title" => "Configstore Device Backups", "mapping" => "index.php:", "url" => "configstore_backups.php", "level" => "1");

        $nav["configstore_archive.php:"] = array("title" => "Configstore Backup Archive", "mapping" => "index.php:", "url" => "configstore_archive.php", "level" => "1");
        $nav["configstore_archive.php:edit"] = array("title" => "Configstore Backup Archive", "mapping" => "index.php:", "url" => "configstore_archive.php", "level" => "1");
        $nav["configstore_archive.php:actions"] = array("title" => "Configstore Backup Archive", "mapping" => "index.php:", "url" => "configstore_archive.php", "level" => "1");
        $nav["configstore_archive.php:viewconfig"] = array("title" => "Configstore Backup Archive", "mapping" => "index.php:", "url" => "configstore_archive.php", "level" => "1");

        $nav["configstore_templates.php:"] = array("title" => "Configstore Backup Templates", "mapping" => "index.php:", "url" => "configstore_templates.php", "level" => "1");
        $nav["configstore_templates.php:edit"] = array("title" => "Configstore Backup Templates", "mapping" => "index.php:", "url" => "configstore_templates.php", "level" => "1");
        $nav["configstore_templates.php:actions"] = array("title" => "Configstore Backup Templates", "mapping" => "index.php:", "url" => "configstore_templates.php", "level" => "1");
        $nav["configstore_templates.php:viewconfig"] = array("title" => "Configstore Backup Templates", "mapping" => "index.php:", "url" => "configstore_templates.php", "level" => "1");

	$nav["configstore_accounts.php:"] = array("title" => "Configstore Device Accounts", "mapping" => "index.php:", "url" => "configstore_accounts.php", "level" => "1");
	$nav["configstore_accounts.php:edit"] = array("title" => "Configstore Device Accounts", "mapping" => "index.php:", "url" => "configstore_accounts.php", "level" => "1");
	$nav["configstore_accounts.php:actions"] = array("title" => "Configstore Device Accounts", "mapping" => "index.php:", "url" => "configstore_accounts.php", "level" => "1");

        $nav["configstore_device_types.php:"] = array("title" => "Configstore Device Types", "mapping" => "index.php:", "url" => "configstore_device_types.php", "level" => "1");
        $nav["configstore_device_types.php:edit"] = array("title" => "Configstore Device Types", "mapping" => "index.php:", "url" => "configstore_device_types.php", "level" => "1");
        $nav["configstore_device_types.php:actions"] = array("title" => "Configstore Device Types", "mapping" => "index.php:", "url" => "configstore_device_types.php", "level" => "1");

        $nav["configstore_sites.php:"] = array("title" => "Configstore Sites", "mapping" => "index.php:", "url" => "configstore_sites.php", "level" => "1");
        $nav["configstore_sites.php:edit"] = array("title" => "Configstore Sites", "mapping" => "index.php:", "url" => "configstore_sites.php", "level" => "1");
        $nav["configstore_sites.php:actions"] = array("title" => "Configstore Sites", "mapping" => "index.php:", "url" => "configstore_sites.php", "level" => "1");

        $nav["configstore_compare.php:"] = array("title" => "Configstore Device Config Compare", "mapping" => "index.php:", "url" => "configstore_compare.php", "level" => "1");

	return $nav;
}

function configstore_show_tab () {
        global $config, $user_auth_realm_filenames;

        $realm_id = 87;
	if ((db_fetch_assoc("SELECT user_auth_realm.realm_id FROM user_auth_realm WHERE user_auth_realm.user_id='" . $_SESSION["sess_user_id"] . "' AND user_auth_realm.realm_id='$realm_id'")) || (empty($realm_id))) {

                if (substr_count($_SERVER["REQUEST_URI"], "configstore_")) {
                        print '<a href="' . $config['url_path'] . 'plugins/configstore/configstore_devices.php"><img src="' . $config['url_path'] . 'plugins/configstore/images/tab_configstore_down.gif" alt="ConfigStore" align="absmiddle" border="0"></a>';
                } else {
                        print '<a href="' . $config['url_path'] . 'plugins/configstore/configstore_devices.php"><img src="' . $config['url_path'] . 'plugins/configstore/images/tab_configstore.gif" alt="ConfigStore" align="absmiddle" border="0"></a>';
                }
        }
}

function configstore_config_arrays () {
	global $user_auth_realms, $user_auth_realm_filenames, $menu, $messages;
	global $config;

	$user_auth_realms[87]='Plugin -> ConfigStore';
	$user_auth_realm_filenames['configstore_devices.php'] = 87;
	$user_auth_realm_filenames['configstore_accounts.php'] = 87;
	$user_auth_realm_filenames['configstore_backups.php'] = 87;
	$user_auth_realm_filenames['configstore_compare.php'] = 87;
        $user_auth_realm_filenames['configstore_archive.php'] = 87;
	$user_auth_realm_filenames['configstore_templates.php'] = 87;
	$user_auth_realm_filenames['configstore_device_types.php'] = 87;
	$user_auth_realm_filenames['configstore_sites.php'] = 87;

        if (isset($_SESSION['configstore_message']) && $_SESSION['configstore_message'] != '') {
                $messages['configstore_message'] = array('message' => $_SESSION['configstore_message'], 'type' => 'info');
        }

        if (!function_exists("__")) {
                function __($text, $domain = "cacti") {
                        return $text;
                }
        }

        $menu2 = array ();
        foreach ($menu as $temp => $temp2 ) {
                $menu2[$temp] = $temp2;
                if ($temp == __('Management')) {
                        $menu2["ConfigStore"]["plugins/configstore/configstore_devices.php"] = "Backup Manager";
                        $menu2["ConfigStore"]["plugins/configstore/configstore_archive.php"] = "Archive (Recycle Bin)";
			$menu2["ConfigStore"]["plugins/configstore/configstore_templates.php"] = "Config Templates";
			$menu2["ConfigStore"]["plugins/configstore/configstore_device_types.php"] = "Device Types";
                }
        }
        $menu = $menu2;
}
